from hashflow.client import Client
from hashflow.constants import ZERO_ADDRESS, PRIV_POOL_CONTRACT, ETH_ADDRESS, ROUTER_CONTRACT
import hashflow.util as utils
import eth_account
import eth_keys

private_key = '0xac0974bec39a17e36ba4a6b4d238ff944bacb478cbed5efcae784d7bf4f2ff80'
signer_private_key = '0x59c6995e998f97a5a0044966f0945389dc9e86dae88c7a8412f4603b6b78690d'
node = 'http://127.0.0.1:8545'
hflow = Client(private_key, network_id=31337, node=node)

# private_key = '0x92faf84d1442867029be862d8e076f0cce38f43052c03ffdd3873aa6e6fc21cd'
# signer_private_key = '0x59c6995e998f97a5a0044966f0945389dc9e86dae88c7a8412f4603b6b78690d'
# node = 'https://mainnet.infura.io/v3/fe673a7397d745348eb876fa3288d2cc'
# signer = '0x2150DD462B496bd5e4A37b5411c13319427BE83E'
# admin = '0x2150DD462B496bd5e4A37b5411c13319427BE83E'

#hflow = Client(private_key, network_id=1, node=node)
name = 'hashflow-7'
symbol = 'NaN'
admin = '0x70997970c51812dc3a010c7d01b50e0d17dc79c8'
signer = '0x70997970c51812dc3a010c7d01b50e0d17dc79c8'
operations = '0xf39fd6e51aad88f6f4ce6ab8827279cfffb92266'
pool = '0x710DD0333d849b46CD4ed9822d6e4a6aE6A9a99f'
eth = ZERO_ADDRESS
liquidity_provider = operations
amount = 500
txid = '0xea513029cdd1d30170053424aa835575d410d8fe6b1f2c4ac17725349c9c9b0c'
expiry = hflow.main.set_expiry(20)
flag = utils.Flag.on
testToken1 = '0x959922bE3CAee4b8Cd9a407cc3ac1C251C2007B1'
testToken2 = '0x3D63c50AD04DD5aE394CAB562b7691DD5de7CF6f'
effective_base_token_amount = 80
base_token_amount = 100
quote_token_amount = 5 

# result1 = hflow.main.factory.set_priv_pool_impl()
# result2 = hflow.main.factory.set_pub_pool_impl()
# result3 = hflow.main.factory.set_hToken_impl()
# print(result1, result2)

#hflow.main.factory.create_pool(name='hash222', symbol='sss', signer=signer)
#hflow.main.factory.create_pool(name='hash35', symbol = 'fgf', signer=signer, privPool = False, permissioned=False)

result4 = hflow.main.factory.get_pools(operations)
print(result4)

#hflow.main.erc20.set_maximum_hashflow_allowance(testToken1)
# result5 = hflow.main.erc20.allowance(testToken1, operations, '0xB7f8BC63BbcaD18155201308C8f3540b07f84F5e')
# print((result5))
#hflow.main.router.add_liquidity_private_pool(result4[0], testToken1, amount)
#hflow.main.pool.list_asset(result4[2], testToken1, cap=1000)
#hflow.main.pool.list_asset(result4[2], eth, cap=1000)
#hflow.main.router.add_liquidity_public_pool(result4[2], eth, amount)
#hflow.main.router.add_liquidity_public_pool(result4[2], testToken1, amount)

# #print(result6)
#result = hflow.main.pool.update_hedging_account(result4[2], [signer], True)
#result7 = hflow.main.router.remove_liquidity_private_pool(result4[0], testToken1, 20, recipient=signer)
# #result8 = hflow.main.router.withdraw_all(result3[0], [eth, testToken1], signer)
#heth = hflow.main.pool.get_hToken(result4[2], eth)

#print(heth)
#hflow.main.erc20.set_maximum_hashflow_allowance(heth)
#hflow.main.governance.set_percent_withdrawal_limit(20)
#hflow.main.router.remove_liquidity_public_pool(result4[2], ETH_ADDRESS, burn_amount=2000)
#hflow.main.erc20.transfer(testToken1, result4[0], amount=100)
hflow.main.pool.transfer_assets(result4[2], eth, recipient=signer, amount=2)
# result9 = hflow.main.pool.reserves(result4[1], ETH_ADDRESS)
# print(result9, 58)
# result10 = hflow.main.pool.payout(result4[1], ETH_ADDRESS, liquidity_provider)
# print(result10)
# result11 = hflow.main.pool.estimate_burn_amount(result4[1], ETH_ADDRESS, 502)
# print(result11)

# # print(hflow.main.get_wallet_balance(signer, token=testToken1))
# # print(hflow.main.pool.reserves(result3[0], testToken1))
# hflow.main.pool.update_fees(result4[1], 200)
# # result = hflow.main.get_wallet_balance(liquidity_provider)
#quote1 = utils.Quote(name, 'TT1', 'ETH', result4[1], liquidity_provider, testToken1, eth, base_token_amount, max_base_token_amount, max_quote_token_amount, txid, expiry, flag, k_value=13)
# quote2 = utils.Quote(result4[1], liquidity_provider, eth, testToken1, base_token_amount, quote_token_amount, txid, expiry, flag, k_value=4)
# # # quote3 = utils.Quote(name, 'TT2', 'TT1', result1[3], liquidity_provider, testToken2, testToken1, base_token_amount, quote_token_amount, txid, expiry, flag, k_value=12)
quote = utils.Quote(pool=result4[2], eoa=ZERO_ADDRESS, trader=operations, effectiveTrader=operations, base_token_address=eth, quote_token_address=testToken1, base_token_amount=10, quote_token_amount=50, fees=0, expiry=expiry, flag=flag, txid=txid, k_value=4, trade_eoa=False)
# # #quote = utils.Quote(pool_name, base_token_name, quote_token_name, pool, trader, base_token_address, quote_token_address, base_token_amount, quote_token_amount, txid, expiry, flag, k_value)
# result = hflow.main.pool.reserves(result4[1], testToken1)
#print(quote)
quote_digest = hflow.main.hash_quote(quote)
#print(quote.__dict__)
signed_quote = utils.sign_digest(quote_digest, signer_private_key)
#print(signed_quote)

#hflow.main.erc20.set_maximum_allowance(result1[3])
#hflow.main.router.trade_single_hop(quote, signed_quote, effective_base_token_amount=10) 
#hflow.main.router.trade(quote2, signed_quote, tradeType=utils.Trade.eth_to_token)
#hflow.main.router.trade(quote3, signed_quote, tradeType=utils.Trade.tokens)
#print(hflow.main.pool.nonce(result3[0], operations))
#print(hflow.main.pool.flag(result3[0], 0))



import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from hashflow.client import Client

OPERATIONS_ADDRESS = '0x70997970c51812dc3a010c7d01b50e0d17dc79c8'
OPERATIONS_PRIVATE_KEY = '0x59c6995e998f97a5a0044966f0945389dc9e86dae88c7a8412f4603b6b78690d'

SIGNER_ADDRESS = '0x90f79bf6eb2c4f870365e785982e1f101e93b906'
SIGNER_PRIVATE_KEY = '0x7c852118294e51e653712a81e05800f419141751be58f605c371e15141b007a6'

LOCALHOST_NETWORK_ID = 31337
LOCALHOST_NODE = 'http://127.0.0.1:8545'

hashflow = Client(
  private_key=OPERATIONS_PRIVATE_KEY,
  network_id=LOCALHOST_NETWORK_ID,
  node=LOCALHOST_NODE
)

def test_private_pool_creation():
  hashflow.main.factory.create_pool(
    name='testpool1',
    signer=SIGNER_ADDRESS,
  )

def test_get_pools():
  pools = hashflow.main.factory.get_pools(OPERATIONS_ADDRESS)
  assert len(pools) == 1


from hashflow.constants import ZERO_ADDRESS
from hashflow.util import Trade, Flag


class Governance(object):

    def __init__(
        self,
        main,
    ):
        self.main = main

        # initialize contracts
        self.governance = self.main.create_hashflow_contract(
            'HashflowGovernance',
            self.main.network_id
        )

    def set_withdrawal_period(
        self,
        withdrawal_period,
        options=None,
    ):

        return self.main.send_eth_transaction(
            method=self.governance.functions.setWithdrawPeriod(
                withdrawal_period
            ),
            options=options
        )

    def set_percent_withdrawal_limit(
        self,
        percent_withdrawal_limit,
        options=None,
    ):

        return self.main.send_eth_transaction(
            method=self.governance.functions.setPercentWithdrawLimit(
                percent_withdrawal_limit
            ),
            options=options
        )

    def update_router_auth_status(
        self,
        router,
        status,
        options=None,
    ):

        return self.main.send_eth_transaction(
            method=self.governance.functions.updateRouterAuthStatus(
                router,
                status,
            ),
            options=options
        )
    
    def get_withdrawal_period(
        self,
        options=None,
    ):

        return self.main.send_eth_transaction(
            method=self.governance.functions.getWithdrawPeriod(
            ),
            options=options
        )

    def get_percent_withdrawal_limit(
        self,
        options=None,
    ):

        return self.main.send_eth_transaction(
            method=self.governance.functions.getPercentWithdrawLimit(
            ),
            options=options
        )

    def router_authorized(
        self,
        options=None,
    ):

        return self.main.send_eth_transaction(
            method=self.governance.functions.getRouterAuthStatus(
            ),
            options=options
        )

    



import eth_keys
import eth_account
from web3 import Web3
from enum import IntEnum


class Trade(IntEnum):
    
    tokens = 1
    eth_to_token = 2
    token_to_eth = 3


class Flag(IntEnum):
 
    off = 0
    on = 1
    na = 2

class Quote(object):

    def __init__(
        self,
        pool,
        eoa,
        trader,
        effectiveTrader,
        base_token_address,
        quote_token_address,
        base_token_amount,
        quote_token_amount,
        fees,
        expiry,
        flag,
        txid,
        trade_eoa,
        k_value=None,
        nonce=None,
    ):

        self.pool = pool
        self.eoa = eoa
        self.trader = trader
        self.effectiveTrader = effectiveTrader
        self.base_token_address = base_token_address
        self.quote_token_address = quote_token_address
        self.base_token_amount = base_token_amount
        self.quote_token_amount = quote_token_amount
        self.fees = fees
        self.expiry = expiry
        self.flag = flag
        self.txid = txid
        self.trade_eoa = trade_eoa
        self.k_value = k_value
        self.nonce = nonce


def strip_hex_prefix(input):
    if input[0:2] == '0x':
        return input[2:]
    else:
        return input


def normalize_private_key(private_key):
    if type(private_key) == str:
        return bytearray.fromhex(strip_hex_prefix(private_key))
    elif type(private_key) == bytearray:
        return private_key
    else:
        raise TypeError('private_key incorrect type')


def private_key_to_address(key):
    eth_keys_key = eth_keys.keys.PrivateKey(key)
    return eth_keys_key.public_key.to_checksum_address()


def sign_digest(digest, private_key):
    result = eth_account.account.Account.signHash(digest, private_key)
    return result['signature'].hex()



import os
import json

from hashflow.constants import MAX_SOLIDITY_UINT

class ERC20(object):

    def __init__(
        self,
        main
    ):
        self.main = main
        this_folder = os.path.dirname(os.path.abspath(__file__))

        deployed_file_path = os.path.join(
            this_folder,
            'deployed.json'
        )
        deployed_addresses = json.load(
            open(deployed_file_path, 'r')
        )

        self.router_address = \
            deployed_addresses['HashflowRouter'][str(self.main.network_id)]['address']

    # -----------------------------------------------------------
    # Transactions
    # -----------------------------------------------------------

    def allowance(
        self,
        token,
        owner,
        spender,
    ):
        contract = self.main.create_contract(
            'IERC20',
            self.main.web3.toChecksumAddress(token),
        )

        result = contract.functions.allowance(
            self.main.web3.toChecksumAddress(owner),
             self.main.web3.toChecksumAddress(spender),
        ).call()

        return result

    def totalSupply(
        self,
        token,
    ):
        contract = self.main.create_contract(
            'IERC20',
            token,
        )

        result = contract.functions.totalSupply().call()

        return result

    def name(
        self,
        token,
    ):
        contract = self.main.create_contract(
            'IERC20',
            token,
        )

        result = contract.functions.name().call()

        return result


    def symbol(
        self,
        token,
    ):
        contract = self.main.create_contract(
            'IERC20',
            token,
        )

        result = contract.functions.symbol().call()

        return result

    def decimals(
        self,
        token,
    ):
        contract = self.main.create_contract(
            'IERC20',
            token,
        )

        result = contract.functions.decimals().call()

        return result

    def set_maximum_hashflow_allowance(
        self,
        token,
        options=None
    ):
        contract = self.main.create_contract(
            'IERC20',
            token,
        )
        return self.main.send_eth_transaction(
            method=contract.functions.approve(
                self.router_address,
                MAX_SOLIDITY_UINT
            ),
            options=options
        )


    def unset_allowance(
        self,
        token,
        options=None
    ):
        contract = self.main.create_contract(
            'IERC20',
            token,
        )
        return self.main.send_eth_transaction(
            method=contract.functions.approve(
                self.router_address,
                0,
            ),
            options=options
        )

    
    def transfer(
        self,
        token,
        receiver,
        amount,
        options=None
    ):
        contract = self.main.create_contract(
            'IERC20',
            token,
        )
        return self.main.send_eth_transaction(
            method=contract.functions.transfer(
                self.main.web3.toChecksumAddress(receiver),
                amount,
            ),
            options=options
        )

    def transfer_from(
        self,
        token,
        spender,
        receiver,
        amount,
        options=None
    ):
        contract = self.main.create_contract(
            'IERC20',
            token,
        )
        return self.main.send_eth_transaction(
            method=contract.functions.transferFrom(
                self.main.web3.toChecksumAddress(spender),
                self.main.web3.toChecksumAddress(receiver),
                amount,
            ),
            options=options
        )




import json

import hashflow.util as utils
from hashflow.main import Main



class Client(object):
  
    def __init__(
        self,
        private_key,
        network_id,
        node=None
    ):
        self.private_key = utils.normalize_private_key(private_key)
        self.public_address = utils.private_key_to_address(self.private_key)
        self.network_id = network_id
        self.main = Main(
            node=node,
            private_key=self.private_key,
            public_address=self.public_address,
            network_id=self.network_id
        )



import os
import json

from hashflow.constants import ZERO_ADDRESS


class Factory(object):

    def __init__(
        self,
        main,
    ):
        self.main = main

        # initialize contracts
        self.factory = self.main.create_hashflow_contract(
            'HashflowFactory',
            self.main.network_id
        )
        this_folder = os.path.dirname(os.path.abspath(__file__))

        deployed_file_path = os.path.join(
            this_folder,
            'deployed.json'
        )
        deployed_addresses = json.load(
            open(deployed_file_path, 'r')
        )
        self.router_address = \
            deployed_addresses['HashflowRouter'][str(self.main.network_id)]['address']
        self.governance_address = \
            deployed_addresses['HashflowGovernance'][str(self.main.network_id)]['address']
    
    def test(
        self,
        options=None
    ):
        '''
        Test connection with the network.
        '''
        result = self.factory.functions.test().call()
        return result

    def create_pool(
        self,
        name,
        signer,
        symbol=None,
        privPool=True,
        permissioned=True,
        options=None
    ):
        '''
        Allows market makers to create a pool.
        '''
        return self.main.send_eth_transaction(
            method=self.factory.functions.createPool(
                name,
                symbol or '',
                self.main.web3.toChecksumAddress(signer),
                self.main.web3.toChecksumAddress(self.governance_address),
                self.main.web3.toChecksumAddress(self.router_address),
                privPool,
                permissioned
            ),
            options=options
        )

   
    def pool_name_taken(
        self,
        name,
    ):
        '''
        Check if the pool name is taken.
        '''
        result = self.factory.functions.poolNameTaken(name).call()
        return result

    
    def get_pools(
        self,
        operations,
    ):
        '''
        Query all pool addresses deployed using the operations key. 
        '''
        result = self.factory.functions.getPools(
            self.main.web3.toChecksumAddress(operations),
        ).call()

        return result

    



class Pool(object):

    def __init__(
        self,
        main,
    ):
        self.main = main

    # -----------------------------------------------------------
    # Transactions
    # -----------------------------------------------------------

    def update_router_permissions(
        self,
        pool,
        router,
        permissions,
        options=None
    ):
        contract = self.main.create_contract(
            'IHashflowPool',
            self.main.web3.toChecksumAddress(pool),
        )
        return self.main.send_eth_transaction(
            method=contract.functions.updateRouterPermissions(
                router, permissions
            ),
            options=options
        )

    def update_signer(
        self,
        signer,
        options=None
    ):

        contract = self.main.create_contract(
            'IHashflowPool',
            self.main.web3.toChecksumAddress(pool),
        )

        return self.main.send_eth_transaction(
            method=contract.functions.updateSigner(
                self.main.web3.toChecksumAddress(signer),
            ),
            options=options
        )

    def update_lp_account(
        self,
        pool,
        whitelist,
        status,
        options=None
    ):
        '''
        Pool owners can authorize/unauthorize whitelist addresses that can 
        contribute assets to the pool. 
        '''

        checksum_whitelist = []
        for lp_account in whitelist:
            checksum_whitelist.append(
                self.main.web3.toChecksumAddress(lp_account))

        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.updateLpAccount(
                checksum_whitelist, status
            ),
            options=options
        )

    def update_migration_pool(
        self,
        current_pool,
        margin_pool,
        options=None
    ):

        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(current_pool),
            'abi/IHashflowSpotPubPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.updateMigrationPool(migration_pool),
            options=options
        )

    def update_migrate_mode(
        self,
        pool,
        migrate_mode,
        options=None
    ):

        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.updateMigrateMode(migrateMode),
            options=options
        )

    def update_hedging_account(
        self,
        pool,
        whitelist,
        status,
        options=None
    ):
        '''
        Pool owners can authorize/unauthorize whitelist addresses that be used to withdraw
        funds
        '''

        checksum_whitelist = []
        for hedging_account in whitelist:
            checksum_whitelist.append(
                self.main.web3.toChecksumAddress(hedging_account))

        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.updateHedgingAccount(
                checksum_whitelist, status
            ),
            options=options
        )

    def update_margin(
        self,
        pool,
        margin,
        options=None
    ):
        '''
        Pool owners update the Opyn's margin pool address in order to trade 
        options.
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowPrivPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.updateMargin(margin),
            options=options
        )

    def approve_margin(
        self,
        pool,
        token,
        options=None
    ):
        '''
        Approve Opyn's margin contract to spend max allowance
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowPrivPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.approveMargin(token),
            options=options
        )

    def list_asset(
        self,
        pool,
        token,
        cap,
        options=None
    ):
        '''
        List a new asset.
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.listAsset(token, cap),
            options=options
        )

    def update_cap(
        self,
        pool,
        token,
        cap,
        options=None
    ):
        '''
        Update an asset's cap.
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.updateCap(token, cap),
            options=options
        )

    def stop_trading(
        self,
        pool,
        stop,
        options=None
    ):

        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.updateTrading(stop),
            options=options
        )

    def update_fees(
        self,
        pool,
        fees,
        options=None
    ):

        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.updateFees(fees),
            options=options
        )

    def transfer_assets(
        self,
        pool,
        token,
        recipient,
        amount,
        options=None
    ):
        '''
        Transfer postive equity balance to CeFi venue to rebalance pools. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.transferAssets(
                self.main.web3.toChecksumAddress(token),
                self.main.web3.toChecksumAddress(recipient),
                amount
            ),
            options=options
        )

    def get_pool_name(
        self,
        pool,
    ):
        '''
        Query pool name. Pool's name is the same as Lp token name. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowPool.json'
        )

        result = contract.functions.getPoolName().call()
        return result

    def get_signer(
        self,
        pool,
    ):
        '''
        Query signer address.
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowPool.json'
        )

        result = contract.functions.getSigner().call()
        return result

    def get_reserves(
        self,
        pool,
        token,
    ):
        '''
        Query pool's reserves for an asset.
        For eth, set token address to ZERO_ADDRESS
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowPool.json'
        )

        result = contract.functions.getReserves(
            self.main.web3.toChecksumAddress(token)
        ).call()
        return result

    def get_hToken(
        self,
        pool,
        token,
    ):
        '''
        Query a token's hToken address associated with the pool. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )

        result = contract.functions.getHToken(
            self.main.web3.toChecksumAddress(token)
        ).call()
        return result

    def get_withdraw_limit(
        self,
        pool,
        token,
    ):
        '''
        Query an account's withdraw limit. W.L is updated each week. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )

        result = contract.functions.getWithdrawLimit(
            self.main.web3.toChecksumAddress(token)
        ).call()
        return result

    def get_cap(
        self,
        pool,
        token,
    ):
        '''
        Query a public pools max capacity for an asset. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )

        result = contract.functions.getCap(
            self.main.web3.toChecksumAddress(token)
        ).call()
        return result

    def asset_listed(
        self,
        pool,
        token,
    ):
        '''
        Check if asset is listed. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )

        result = contract.functions.assetListed(
            self.main.web3.toChecksumAddress(token)
        ).call()
        return result

    def get_net_payout(
        self,
        pool,
        token,
    ):
        '''
        Query pool's total payout. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )

        result = contract.functions.getNetPayout(
            self.main.web3.toChecksumAddress(token)
        ).call()
        return result

    def get_withdrawal_timestamp(
        self,
        pool,
        token,
    ):
        '''
        Query pool's total payout. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )

        result = contract.functions.getWithdrawalTimestamp(
            self.main.web3.toChecksumAddress(token)
        ).call()
        return result

    def get_payout(
        self,
        pool,
        token,
        liquidity_provider,
    ):
        '''
        Query a liquidity provider's total payout. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )

        result = contract.functions.computeLpPayout(
            self.main.web3.toChecksumAddress(token),
            self.main.web3.toChecksumAddress(liquidity_provider)
        ).call()
        return result

    def estimate_burn_amount(
        self,
        pool,
        token,
        withdrawAmount,
    ):
        '''
        Query a liquidity provider's total payout. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )

        result = contract.functions.computeBurnAmount(
            self.main.web3.toChecksumAddress(token),
            withdrawAmount
        ).call()
        return result

    def get_nonce(
        self,
        pool,
        account,
    ):
        '''
        Query nonce associated with an account.
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowPool.json'
        )

        result = contract.functions.getNonce(
            self.main.web3.toChecksumAddress(account)
        ).call()
        return result

    def get_flag(
        self,
        pool,
        k_value,
    ):
        '''
        Query flag associated with a k-value for a pool.
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowPool.json'
        )

        result = contract.functions.getFlag(k_value).call()
        return result

    def get_migration_pool(
        self,
        pool,
    ):
        '''
        Query migration pool address. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )

        result = contract.functions.getMigrationPool().call()
        return result

    def get_migration_mode(
        self,
        pool,
    ):
        '''
        Check if pool is in migration mode. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowSpotPubPool.json'
        )

        result = contract.functions.getMigrationMode().call()
        return result

    def get_governance(
        self,
        pool,
    ):
        '''
        Query governance contract address. 
        '''
        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowPool.json'
        )

        result = contract.functions.getGovernance().call()
        return result



from setuptools import setup, find_packages

LONG_DESCRIPTION = open('README.md', 'r').read()

REQUIREMENTS = open('requirements.txt', 'r').read().split('\n')

setup(
    name='hashflow-python',
    version='10.0.0',
    packages=find_packages(),
    package_data={
        'hashflow': ['abi/*.json'],
    },
    description='Python SDK to interact with hashflow Smart Contracts',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url='',
    author='H-Protocol Inc',
    license='Apache 2.0',
    author_email='varun@hashflow.com',
    install_requires=REQUIREMENTS,
    keywords='hashflow exchange api defi ethereum eth',
    classifiers=[],
)


import json
import os
import time
from web3 import Web3
import hashflow.constants as consts
from hashflow.factory import Factory
from hashflow.router import Router
from hashflow.governance import Governance
from hashflow.erc20 import ERC20
from hashflow.pool import Pool
from hashflow.util import Flag


class Main(object):

    def __init__(
        self,
        node,
        private_key,
        public_address,
        network_id
    ):
        self.web3 = Web3(None if node is None else Web3.HTTPProvider(node))
        self.private_key = private_key
        self.public_address = public_address
        self.min_nonce = self.web3.eth.getTransactionCount(self.public_address)
        self.network_id = network_id

        self.factory = Factory(self)
        self.router = Router(self)
        self.governance = Governance(self)
        self.erc20 = ERC20(self)
        self.pool = Pool(self)

    # -----------------------------------------------------------
    # Helper Functions
    # -----------------------------------------------------------

    def send_eth_transaction(
        self,
        method,
        options=None
    ):
        if options is None:
            options = dict()
        if 'from' not in options:
            options['from'] = self.public_address
        if 'nonce' not in options:
            options['nonce'] = max(
                self.min_nonce,
                self.web3.eth.getTransactionCount(self.public_address)
            )
        if 'maxFeePerGas' not in options:
            options['maxFeePerGas'] = consts.DEFAULT_MAX_GAS_FEE

        if 'maxPriorityFeePerGas' not in options:
            options['maxPriorityFeePerGas'] = consts.DEFAULT_MAX_PRIORITY_FEE
        # if 'gasPrice' not in options:
        #     try:
        #         options['gasPrice'] = \
        #             self.web3.eth.gasPrice + consts.DEFAULT_GAS_PRICE_ADDITION
        #     except Exception:
        #         options['gasPrice'] = consts.DEFAULT_GAS_PRICE
        if 'value' not in options:
            options['value'] = 0
        if 'gas' not in options:
            try:
                options['gas'] = int(
                    method.estimateGas(options) *
                    consts.DEFAULT_GAS_MULTIPLIER
                )
            except Exception:
                options['gas'] = consts.DEFAULT_GAS_AMOUNT
        self.min_nonce = max(self.min_nonce, options['nonce'] + 1)
        tx = method.buildTransaction(options)
        signed_tx = self.web3.eth.account.sign_transaction(
            tx, self.private_key)
        return self.web3.eth.send_raw_transaction(signed_tx.rawTransaction).hex()

    def create_hashflow_contract(
        self,
        name,
        network_id
    ):
        this_folder = os.path.dirname(os.path.abspath(__file__))
        abi_file_path = os.path.join(
            this_folder,
            'abi/I{}.json'.format(name)
        )
        deployed_file_path = os.path.join(
            this_folder,
            'deployed.json'
        )
        deployed_addresses = json.load(
            open(deployed_file_path, 'r')
        )
        contract_address = deployed_addresses[name][str(network_id)]['address']
        return self.web3.eth.contract(
            address=contract_address,
            abi=json.load(open(abi_file_path, 'r'))
        )

    def create_contract(
        self,
        name,
        address
    ):
        this_folder = os.path.dirname(os.path.abspath(__file__))
        abi_file_path = os.path.join(
            this_folder,
            'abi/{}.json'.format(name)
        )
        return self.web3.eth.contract(
            address=address,
            abi=json.load(open(abi_file_path, 'r'))
        )

    def get_receipt(
        self,
        tx_hash
    ):
        return self.web3.eth.waitForTransactionReceipt(tx_hash)

    def hash_quote(self, quote):
        '''
        Generate the keccak hash of quote object.
        '''

        if quote.flag == Flag.off:
            self.k_value_or_nonce = quote.k_value
        if quote.flag == Flag.on:
            self.k_value_or_nonce = quote.k_value
        if quote.flag == Flag.na:
            self.k_value_or_nonce = quote.nonce

        return Web3.solidityKeccak(
            [
                "address",
                "address",
                "address",
                "address",
                "uint256",
                "uint256",
                "uint256",
                "uint256",
                "uint256",
                "bytes32"
            ],
            [self.web3.toChecksumAddress(quote.pool),
                self.web3.toChecksumAddress(quote.trader),
                self.web3.toChecksumAddress(quote.base_token_address),
                self.web3.toChecksumAddress(quote.quote_token_address),
                quote.base_token_amount,
                quote.quote_token_amount,
                quote.fees,
                self.k_value_or_nonce,
                quote.expiry,
                quote.txid
             ]
        ).hex()

    def hash_quote_eoa(self, quote):
        '''
        Generate the keccak hash of quote object.
        '''

        if quote.flag == Flag.off:
            self.k_value_or_nonce = quote.k_value
        if quote.flag == Flag.on:
            self.k_value_or_nonce = quote.k_value
        if quote.flag == Flag.na:
            self.k_value_or_nonce = quote.nonce

        return Web3.solidityKeccak(
            [
                "address",
                "address",
                "address",
                "address",
                "uint256",
                "uint256",
                "uint256",
                "uint256",
                "uint256",
                "bytes32"
            ],
            [self.web3.toChecksumAddress(quote.pool),
                self.web3.toChecksumAddress(quote.trader),
                self.web3.toChecksumAddress(quote.eoa),
                self.web3.toChecksumAddress(quote.base_token_address),
                self.web3.toChecksumAddress(quote.quote_token_address),
                quote.base_token_amount,
                quote.quote_token_amount,
                quote.fees,
                self.k_value_or_nonce,
                quote.expiry,
                quote.txid
             ]
        ).hex()

    def get_wallet_balance(
        self,
        account,
        token=None
    ):
        '''
        Gets the on-chain balance of a users wallet for some asset.
        '''
        if token is None:
            balance = self.web3.eth.getBalance(
                self.web3.toChecksumAddress(account))
        else:
            contract = self.create_contract(
                'IERC20',
                self.web3.toChecksumAddress(token)
            )
            balance = contract.functions.balanceOf(
                self.web3.toChecksumAddress(account)
            ).call()
        return balance

    def get_block_number(self):
        '''
        Gets the latest block number.
        '''
        return self.web3.eth.get_block_number()

    def set_expiry(self, expiry):
        '''
        Sets the quote expiry time (in seconds).
        '''
        result = int(time.time()) + expiry
        return result

    def convert_to_decimals(
        self,
        amount,
        token=None
    ):
        '''
        Convert asset amount to decimals.
        '''
        if token is None:
            decimals = 18
        else:
            contract = self.create_contract(
                'IERC20',
                token,
            )
            decimals = contract.functions.decimals().call()

        return amount * (10**18)

    def convert_from_decimals(
        self,
        amount,
        token=None
    ):
        '''
        Convert asset amount from decimals.
        '''
        if token is None:
            decimals = 18
        else:
            contract = self.create_contract(
                'IERC20',
                token,
            )
            decimals = contract.functions.decimals().call()

        return amount/(10**18)



from hashflow.constants import ZERO_ADDRESS
from hashflow.util import Trade, Flag


class Router(object):

    def __init__(
        self,
        main,
    ):
        self.main = main

        # initialize contracts
        self.router = self.main.create_hashflow_contract(
            'HashflowRouter',
            self.main.network_id
        )

    def add_liquidity_private_pool(
        self,
        pool,
        token,
        amount,
        options=None,
    ):
        '''
        Add liquidity into private pool.
        '''

        if token == ZERO_ADDRESS:
            if options is None:
                options = dict(
                    value=amount
                )
            else:
                options['value'] = amount

        return self.main.send_eth_transaction(
            method=self.router.functions.addLiquidityPrivatePool(
                self.main.web3.toChecksumAddress(pool),
                self.main.web3.toChecksumAddress(token),
                amount,
            ),
            options=options
        )

    def add_liquidity_public_pool(
        self,
        pool,
        token,
        amount,
        options=None,
    ):
        '''
        Add liquidity into public pool.
        '''

        if token == ZERO_ADDRESS:
            if options is None:
                options = dict(
                    value=amount
                )
            else:
                options['value'] = amount

        return self.main.send_eth_transaction(
            method=self.router.functions.addLiquidityPublicPool(
                self.main.web3.toChecksumAddress(pool),
                self.main.web3.toChecksumAddress(token),
                amount,
            ),
            options=options
        )

    def remove_liquidity_private_pool(
        self,
        pool,
        token,
        amount,
        recipient=ZERO_ADDRESS,
        options=None,
    ):
        '''
        Remove liquidity from the private pool.
        '''
        _recipient = ZERO_ADDRESS
        if recipient != ZERO_ADDRESS:
            _recipient = recipient

        contract = self.main.create_contract(
            self.main.web3.toChecksumAddress(pool),
            'abi/IHashflowPrivPool.json'
        )
        return self.main.send_eth_transaction(
            method=contract.functions.withdraw(
                self.main.web3.toChecksumAddress(token),
                self.main.web3.toChecksumAddress(_recipient),
                amount,
            ),
            options=options,
        )

    def remove_liquidity_public_pool(
        self,
        pool,
        token,
        burn_amount,
        options=None,
    ):
        '''
        Remove liquidity from public pool.
        '''

        return self.main.send_eth_transaction(
            method=self.router.functions.removeLiquidityPublicPool(
                self.main.web3.toChecksumAddress(pool),
                self.main.web3.toChecksumAddress(token),
                burn_amount,
            ),
            options=options,
        )

    def trade_single_hop(
        self,
        quote_data,
        signed_quote,
        effective_base_token_amount=None,
        options=None,
    ):

        if effective_base_token_amount is None:
            effective_base_token_amount = quote_data.base_token_amount

        if quote_data.base_token_address == ZERO_ADDRESS:
            if options is None:
                options = dict(
                    value=(effective_base_token_amount)
                )
            else:
                options['value'] = (
                    effective_base_token_amount
                )

        if quote_data.flag == Flag.off:
            self.k_value_or_nonce = quote_data.k_value
        if quote_data.flag == Flag.on:
            self.k_value_or_nonce = quote_data.k_value
        if quote_data.flag == Flag.na:
            self.k_value_or_nonce = quote_data.nonce

        quote = {
            'pool': self.main.web3.toChecksumAddress(quote_data.pool),
            'eoa': self.main.web3.toChecksumAddress(
                        quote_data.eoa),
            'trader': self.main.web3.toChecksumAddress(
                        quote_data.trader),
            'effectiveTrader': self.main.web3.toChecksumAddress(
                        quote_data.effectiveTrader),
            'baseToken': self.main.web3.toChecksumAddress(
                    quote_data.base_token_address),
            'quoteToken': self.main.web3.toChecksumAddress(
                        quote_data.quote_token_address),
            'effectiveBaseTokenAmount': effective_base_token_amount,
            'maxBaseTokenAmount': quote_data.base_token_amount,
            'maxQuoteTokenAmount': quote_data.quote_token_amount,
            'fees': quote_data.fees,
            'quoteExpiry': quote_data.expiry,
            'kValueOrNonce': self.k_value_or_nonce,
            'tradeEOA': quote_data.trade_eoa,
            'flag': quote_data.flag,
            'txid': quote_data.txid,
            'signedQuote': signed_quote
        }

        return self.main.send_eth_transaction(
            method=self.router.functions.tradeSingleHop(quote),
            options=options
        )

    def set_weth(
        self,
        weth,
        options=None,
    ):
        '''
        Update weth address in the router.
        '''

        return self.main.send_eth_transaction(
            method=self.router.functions.setWeth(
                self.main.web3.toChecksumAddress(weth),
            ),
            options=options
        )
